@@ -1,7 +1,7 @@
 import django
 from datetime import datetime
 from django.db import models, transaction
-from django.db.models.query import EmptyQuerySet
+from django.db.models.query import QuerySet, EmptyQuerySet
 from django.core.exceptions import ImproperlyConfigured
 from .exceptions import ObjectSetError
 from .decorators import cached_property
@@ -36,8 +36,14 @@ class ObjectSet(models.Model):
         # Assume this is a list, tuple, queryset of objects
         if args and hasattr(args[0], '__iter__'):
             save = kwargs.pop('save', False)
-            self._pending = args[0]
-            args = args[1:]
+            args = list(args)
+            queryset = args.pop(0)
+
+            # Create a queryset if this is a list of tuple of instances
+            if not isinstance(queryset, QuerySet):
+                pks = [x.pk for x in queryset]
+                queryset = self._object_class.objects.filter(pk__in=pks)
+            self._pending = queryset
 
         super(ObjectSet, self).__init__(*args, **kwargs)
 
@@ -83,20 +89,24 @@ class ObjectSet(models.Model):
     def __iand__(self, other):
         "Performs an inplace intersection of this set and `other`."
         self._pending = other._objects & self._objects
+        return self
 
     def __ior__(self, other):
         "Performs and inplace union of this set and `other`."
         self._pending = other._objects | self._objects
+        return self
 
     def __ixor__(self, other):
         "Performs an inplace exclusive union of this set and `other`."
         excluded = models.Q(pk__in=(other._objects & self._objects))
         self._pending = (other._objects | self._objects).exclude(excluded)
+        return self
 
     def __isub__(self, other):
         "Inplace removal of objects from this set that are in `other`."
         excluded = models.Q(pk__in=other._objects)
         self._pending = self._objects.exclude(excluded)
+        return self
 
     @cached_property
     def _set_object_rel(self):
@@ -172,7 +182,7 @@ class ObjectSet(models.Model):
         objects = self._object_class.objects.all()
         pks = self._set_objects(include_removed=False)\
             .values_list('{0}__pk'.format(self._through_object_rel))
-        return objects.filter(pk__in=pks)
+        return objects.filter(pk__in=pks) | self._pending
 
     def _set_objects(self, include_removed=True):
         "Returns a queryset of set objects."
@@ -242,12 +252,12 @@ class ObjectSet(models.Model):
 
         # Handle pending data after the set has been saved
         if self._pending is not None and not isinstance(self._pending, EmptyQuerySet):
-            pending = self._pending
+            pending = list(self._pending.only('pk'))
             self._pending = self._object_class.objects.none()
             if new and BULK_SUPPORTED:
                 self.bulk(pending)
             else:
-                self.update(pending)
+                self.replace(pending)
 
     @transaction.commit_on_success
     def bulk(self, objs, added=False):
